/********************************************************************************************************
Classic "PONG" game for Arduino. Written for display from Nokia-3410 (physical resolution is 96x65, but
IC PCF8812 has 102x72. Game is controlled with 2 pots 10K and one button for start game. Pins for
connections are defined below. Added some sounds (connect speaker through resistor 100ohms to defined
pin).
Pots are connected: left pin to +5V, middle pin to Analog pin and right pin to GND
Button is connected: first pin to +5V, second pin to Digital pin. Digital pin is connected to GND
through resistor 10K.
There are defined pins PIN_GND and PIN_PLUS, these pins are not necessary, you can power the display
from normal +5V and GND pins :-)

No aditional library is needed.

Some remarks are in my native language (Czech)
*********************************************************************************************************/
#define PIN_GND     2
#define PIN_RESET   3
#define PIN_SCE     4
#define PIN_DC      5
#define PIN_SDIN    6
#define PIN_SCLK    7
#define PIN_PLUS    8
#define PIN_P1     A0
#define PIN_P2     A1
#define PIN_REPRO   9
#define PIN_BUTTON 10

#define LCD_C     LOW
#define LCD_D     HIGH

#define LCD_X     102
#define LCD_Y     72

#define LCD_CMD   0

//ASCII table with semigraphic symbols
static const byte numbers[][5] =
{ //pseudografika
 {0x00, 0x00, 0xff, 0xff, 0xff} // 11 I vpravo
,{0xff, 0xff, 0xff, 0x00, 0x00} // 12 I vlevo
,{0xf0, 0xf0, 0xf0, 0xf0, 0xf0} // 13 _ nahoře
,{0x0f, 0x0f, 0x0f, 0x0f, 0x0f} // 14 _ dole
,{0x00, 0x00, 0x0f, 0x0f, 0x0f} // 15 . vpravo dole
,{0x0f, 0x0f, 0x0f, 0x00, 0x00} // 16 . vlevo dole
,{0x00, 0x00, 0xf0, 0xf0, 0xf0} // 17 . vpravo nahoře
,{0xf0, 0xf0, 0xf0, 0x00, 0x00} // 18 . vlevo nahoře
,{0x0f, 0x0f, 0xff, 0xf0, 0xf0} // 19 /
,{0xf0, 0xf0, 0xff, 0x0f, 0x0f} // 1a \
,{0xff, 0xff, 0xff, 0x0f, 0x0f} // 1b L
,{0x0f, 0x0f, 0xff, 0xff, 0xff} // 1c obrácené L
,{0xff, 0xff, 0xff, 0xf0, 0xf0} // 1d L hlavou dolů
,{0xf0, 0xf0, 0xff, 0xff, 0xff} // 1e obrácené L hlavou dolů
,{0xff, 0xff, 0xff, 0xff, 0xff} // 1f plný čtverec
  //normální ASCII
,{0x00, 0x00, 0x00, 0x00, 0x00} // 20  
,{0x00, 0x00, 0x5f, 0x00, 0x00} // 21 !
,{0x00, 0x07, 0x00, 0x07, 0x00} // 22 "
,{0x14, 0x7f, 0x14, 0x7f, 0x14} // 23 #
,{0x24, 0x2a, 0x7f, 0x2a, 0x12} // 24 $
,{0x23, 0x13, 0x08, 0x64, 0x62} // 25 %
,{0x36, 0x49, 0x55, 0x22, 0x50} // 26 &
,{0x00, 0x05, 0x03, 0x00, 0x00} // 27 '
,{0x00, 0x1c, 0x22, 0x41, 0x00} // 28 (
,{0x00, 0x41, 0x22, 0x1c, 0x00} // 29 )
,{0x14, 0x08, 0x3e, 0x08, 0x14} // 2a *
,{0x08, 0x08, 0x3e, 0x08, 0x08} // 2b +
,{0x00, 0x50, 0x30, 0x00, 0x00} // 2c ,
,{0x08, 0x08, 0x08, 0x08, 0x08} // 2d -
,{0x00, 0x60, 0x60, 0x00, 0x00} // 2e .
,{0x20, 0x10, 0x08, 0x04, 0x02} // 2f /
,{0x7f, 0x41, 0x41, 0x41, 0x7f} // 30 0
,{0x00, 0x00, 0x7f, 0x00, 0x00} // 31 1
,{0x79, 0x49, 0x49, 0x49, 0x4f} // 32 2
,{0x49, 0x49, 0x49, 0x49, 0x7f} // 33 3
,{0x0f, 0x08, 0x08, 0x08, 0x7f} // 34 4
,{0x4f, 0x49, 0x49, 0x49, 0x79} // 35 5
,{0x7f, 0x49, 0x49, 0x49, 0x79} // 36 6
,{0x01, 0x01, 0x01, 0x01, 0x7f} // 37 7
,{0x7f, 0x49, 0x49, 0x49, 0x7f} // 38 8
,{0x4f, 0x49, 0x49, 0x49, 0x7f} // 39 9
,{0x00, 0x36, 0x36, 0x00, 0x00} // 3a :
,{0x00, 0x56, 0x36, 0x00, 0x00} // 3b ;
,{0x08, 0x14, 0x22, 0x41, 0x00} // 3c <
,{0x14, 0x14, 0x14, 0x14, 0x14} // 3d =
,{0x00, 0x41, 0x22, 0x14, 0x08} // 3e >
,{0x02, 0x01, 0x51, 0x09, 0x06} // 3f ?
,{0x32, 0x49, 0x79, 0x41, 0x3e} // 40 @
,{0x7e, 0x11, 0x11, 0x11, 0x7e} // 41 A
,{0x7f, 0x49, 0x49, 0x49, 0x36} // 42 B
,{0x3e, 0x41, 0x41, 0x41, 0x22} // 43 C
,{0x7f, 0x41, 0x41, 0x22, 0x1c} // 44 D
,{0x7f, 0x49, 0x49, 0x49, 0x41} // 45 E
,{0x7f, 0x09, 0x09, 0x09, 0x01} // 46 F
,{0x3e, 0x41, 0x49, 0x49, 0x7a} // 47 G
,{0x7f, 0x08, 0x08, 0x08, 0x7f} // 48 H
,{0x00, 0x41, 0x7f, 0x41, 0x00} // 49 I
,{0x20, 0x40, 0x41, 0x3f, 0x01} // 4a J
,{0x7f, 0x08, 0x14, 0x22, 0x41} // 4b K
,{0x7f, 0x40, 0x40, 0x40, 0x40} // 4c L
,{0x7f, 0x02, 0x0c, 0x02, 0x7f} // 4d M
,{0x7f, 0x04, 0x08, 0x10, 0x7f} // 4e N
,{0x3e, 0x41, 0x41, 0x41, 0x3e} // 4f O
,{0x7f, 0x09, 0x09, 0x09, 0x06} // 50 P
,{0x3e, 0x41, 0x51, 0x21, 0x5e} // 51 Q
,{0x7f, 0x09, 0x19, 0x29, 0x46} // 52 R
,{0x46, 0x49, 0x49, 0x49, 0x31} // 53 S
,{0x01, 0x01, 0x7f, 0x01, 0x01} // 54 T
,{0x3f, 0x40, 0x40, 0x40, 0x3f} // 55 U
,{0x1f, 0x20, 0x40, 0x20, 0x1f} // 56 V
,{0x3f, 0x40, 0x38, 0x40, 0x3f} // 57 W
,{0x63, 0x14, 0x08, 0x14, 0x63} // 58 X
,{0x07, 0x08, 0x70, 0x08, 0x07} // 59 Y
,{0x61, 0x51, 0x49, 0x45, 0x43} // 5a Z
,{0x00, 0x7f, 0x41, 0x41, 0x00} // 5b [
,{0x02, 0x04, 0x08, 0x10, 0x20} // 5c \
,{0x00, 0x41, 0x41, 0x7f, 0x00} // 5d ]
,{0x04, 0x02, 0x01, 0x02, 0x04} // 5e ^
,{0x40, 0x40, 0x40, 0x40, 0x40} // 5f _
,{0x00, 0x01, 0x02, 0x04, 0x00} // 60 `
,{0x20, 0x54, 0x54, 0x54, 0x78} // 61 a
,{0x7f, 0x48, 0x44, 0x44, 0x38} // 62 b
,{0x38, 0x44, 0x44, 0x44, 0x20} // 63 c
,{0x38, 0x44, 0x44, 0x48, 0x7f} // 64 d
,{0x38, 0x54, 0x54, 0x54, 0x18} // 65 e
,{0x08, 0x7e, 0x09, 0x01, 0x02} // 66 f
,{0x0c, 0x52, 0x52, 0x52, 0x3e} // 67 g
,{0x7f, 0x08, 0x04, 0x04, 0x78} // 68 h
,{0x00, 0x44, 0x7d, 0x40, 0x00} // 69 i
,{0x20, 0x40, 0x44, 0x3d, 0x00} // 6a j 
,{0x7f, 0x10, 0x28, 0x44, 0x00} // 6b k
,{0x00, 0x41, 0x7f, 0x40, 0x00} // 6c l
,{0x7c, 0x04, 0x18, 0x04, 0x78} // 6d m
,{0x7c, 0x08, 0x04, 0x04, 0x78} // 6e n
,{0x38, 0x44, 0x44, 0x44, 0x38} // 6f o
,{0x7c, 0x14, 0x14, 0x14, 0x08} // 70 p
,{0x08, 0x14, 0x14, 0x18, 0x7c} // 71 q
,{0x7c, 0x08, 0x04, 0x04, 0x08} // 72 r
,{0x48, 0x54, 0x54, 0x54, 0x20} // 73 s
,{0x04, 0x3f, 0x44, 0x40, 0x20} // 74 t
,{0x3c, 0x40, 0x40, 0x20, 0x7c} // 75 u
,{0x1c, 0x20, 0x40, 0x20, 0x1c} // 76 v
,{0x3c, 0x40, 0x30, 0x40, 0x3c} // 77 w
,{0x44, 0x28, 0x10, 0x28, 0x44} // 78 x
,{0x0c, 0x50, 0x50, 0x50, 0x3c} // 79 y
,{0x44, 0x64, 0x54, 0x4c, 0x44} // 7a z
,{0x00, 0x08, 0x36, 0x41, 0x00} // 7b {
,{0x00, 0x00, 0x7f, 0x00, 0x00} // 7c |
,{0x00, 0x41, 0x36, 0x08, 0x00} // 7d }
,{0x02, 0x01, 0x02, 0x04, 0x02} // 7e ←
,{0x00, 0x06, 0x09, 0x09, 0x06} // 7f °
};

//some variables for game
int scorea=0;
int scoreb=0;
int micekX;
int micekY;
int dX=2;
int dY=-2;
byte mapa[96*9];
int palka1;
int palka2;
int palkaold1;
int palkaold2;
int ingame=1;

//send characters to display
void lcdCharacter(char character) {
  for (int index = 0; index < 5; index++)
  {
    lcdWrite(LCD_D, numbers[character - 0x11][index]);
  }
  if(character>0x20) {  //for semigraphic symbols remove added 1-dot space
  lcdWrite(LCD_D, 0x00);
  }
}

//clear display, fill buffer for court
void lcdClear(void)
{
  for (int index = 0; index < LCD_X * LCD_Y / 8; index++)
  {
    lcdWrite(LCD_D, 0x00);
    mapa[index]=0;
  }
}

//initialise display
void lcdInitialise(void) {
  pinMode(PIN_GND, OUTPUT);
  digitalWrite(PIN_GND, LOW);
  pinMode(PIN_PLUS, OUTPUT);
  digitalWrite(PIN_PLUS, HIGH);
  pinMode(PIN_SCE, OUTPUT);
  pinMode(PIN_RESET, OUTPUT);
  pinMode(PIN_DC, OUTPUT);
  pinMode(PIN_SDIN, OUTPUT);
  pinMode(PIN_SCLK, OUTPUT);
  digitalWrite(PIN_RESET, LOW);
 // delay(1);
  digitalWrite(PIN_RESET, HIGH);
  lcdWrite(LCD_CMD, 0x21);  // LCD Extended Commands.
  lcdWrite(LCD_CMD, 0xBF);  // Set LCD Vop (Contrast). //B1
  lcdWrite(LCD_CMD, 0x04);  // Set Temp coefficent. //0x04
  lcdWrite(LCD_CMD, 0x14);  // LCD bias mode 1:48. //0x13
  lcdWrite(LCD_CMD, 0x0C);  // LCD in normal mode. 0x0d for inverse
  lcdWrite(LCD_CMD, 0x20);
  lcdWrite(LCD_CMD, 0x0C);
}

//create court
void lcdMapa() {
  int a;
  gotoXY(0,0);
  for(a=0; a<47; a++) {
    mapa[a]=0x01;
    lcdWrite(LCD_D,mapa[a]);
    }
  mapa[a]=0x67;
  lcdWrite(LCD_D,mapa[a]);
  for(a=48; a<96; a++) {
    mapa[a]=0x01;
    lcdWrite(LCD_D,mapa[a]);
    }
  for(a=1; a<8; a++) {
    gotoXY(47,a);
    mapa[96*a+47]=0x66;
    lcdWrite(LCD_D,mapa[96*a+47]);
    }
  gotoXY(0,8);
  for(a=0; a<96; a++) {
    mapa[a]=0x01;
    lcdWrite(LCD_D,mapa[a]);
    }
  mapa[47]=0x67;
} 

//write score to display
void lcdScore(int sca, int scb) {
  int posX, posY, sc0, sc1;
  unsigned long pom;
  sc0=sca/10;
  sc1=sca-sc0*10;
  posX=33;
  posY=3;
  for(int a=0; a<5; a++) {
    pom=numbers[sc0+31][a];
    twoBytes(pom, posX, posY);
    posX++;
  }
  posX=39;
  for(int a=0; a<5; a++) {
    pom=numbers[sc1+31][a];
    twoBytes(pom, posX, posY);
    posX++;
  }
  sc0=scb/10;
  sc1=scb-sc0*10;
  posX=51;
  for(int a=0; a<5; a++) {
    pom=numbers[sc0+31][a];
    twoBytes(pom, posX, posY);
    posX++;
  }
  posX=57;
  for(int a=0; a<5; a++) {
    pom=numbers[sc1+31][a];
    twoBytes(pom, posX, posY);
    posX++;
  }
}  

//draw ball
void lcdMicek(int posX, int posY) {
  unsigned long pom;
  pom=0x0e;
  twoBytes(pom, posX, posY);
  posX++;
  pom=0x1f;
  twoBytes(pom, posX, posY);
  posX++;
  pom=0x1f;
  twoBytes(pom, posX, posY);
  posX++;
  pom=0x1f;
  twoBytes(pom, posX, posY);
  posX++;
  pom=0x0e;
  twoBytes(pom, posX, posY);
}  

//translate physical Y to page and bitshift, transform 2byte to max 3byte, send to display
void twoBytes(unsigned long obsah, int posX, int posY) {
  byte num1, num2, num3;
  byte pgY;
  pgY=posY/8;
  posY=posY%8;
  obsah<<=posY;
  num1=obsah&0xff;
  num2=(obsah>>8)&0xff;
  num3=(obsah>>16)&0xff;
  gotoXY(posX,pgY);
  mapa[96*pgY+posX]^=num1;
  lcdWrite(LCD_D,mapa[96*pgY+posX]);
  if(num2!=0||num3!=0) {
  pgY++;
  gotoXY(posX,pgY);
  mapa[96*pgY+posX]^=num2;
  lcdWrite(LCD_D,mapa[96*pgY+posX]);
  }
  if(num3!=0) {
  pgY++;
  gotoXY(posX,pgY);
  mapa[96*pgY+posX]^=num3;
  lcdWrite(LCD_D,mapa[96*pgY+posX]);
  }
}
  
//print string to display
void lcdString(char *characters) {
  while (*characters)
  {
    lcdCharacter(*characters++);
  }
}

//add score to players
void addscore(int score) {
  lcdScore(scorea,scoreb);
  tone(PIN_REPRO,440,400);
  switch(score) {
    case 1:
      scorea++;
      ingame=2;
      if(scorea>10) {ingame=3;}
      dX=-2;
      dY=-2;
      micekX=85;
      micekY=palka2+4;
      break;
    case 2:
      scoreb++;
      ingame=1;
      if(scoreb>10) {ingame=3;}
      dX=2;
      dY=-2;
      micekX=5;
      micekY=palka1+4;
      break;
  }
  lcdScore(scorea,scoreb);
  delay(600);
}

//draw bats      
void lcdPalka(int palka1, int palka2) {
  unsigned long pom=0xfff;
  twoBytes(pom,2,palka1);
  twoBytes(pom,3,palka1);
  twoBytes(pom,92,palka2);
  twoBytes(pom,93,palka2);
  pom=0x7fe;
  twoBytes(pom,4,palka1);
  twoBytes(pom,91,palka2);
}

//subroutine for start button
int lcdButton() {
  int button;
  button=digitalRead(PIN_BUTTON);
  switch(button) {
    case HIGH:
    if(ingame==3) {
      scorea=0;
      scoreb=0;
      lcdClear();
      lcdMapa();
      lcdScore(scorea,scoreb);
      micekX=5;
      micekY=palka1+4;
    }  
    ingame=0;
    break;
    case LOW:
    switch(ingame) {
      case 1:
        micekY=palka1+4;
        break;
      case 2:
        micekY=palka2+4;
        break;
      case 3:
        micekX=5;
        micekY=palka1+4;
        gotoXY(21,4);
        lcdString("GAME");
        gotoXY(51,4);
        lcdString("OVER");
        break;
    } 
  }
  return ingame;
}

//send byte to display
void lcdWrite(byte dc, byte data) {
  digitalWrite(PIN_DC, dc);
  digitalWrite(PIN_SCE, LOW);
  shiftOut(PIN_SDIN, PIN_SCLK, MSBFIRST, data);
  digitalWrite(PIN_SCE, HIGH);
}

// gotoXY routine to position cursor 
// x - range: 0 to 95
// y - range: 0 to 8

void gotoXY(int x, int y) {
  lcdWrite( 0, 0x80 | x);  // Column.
  lcdWrite( 0, 0x40 | y);  // Row.  
}

void setup(void) {
  pinMode(PIN_BUTTON,INPUT);
  lcdInitialise();
  lcdClear();
  gotoXY(17,0);
  lcdString("Let's Play");
  //print logo PONG from semigraphic symbols
  gotoXY(17,1);
  char logo1[13]={0x13,0x13,0x20,0x17,0x13,0x20,0x18,0x20,0x18,0x17,0x13,0x20};
  lcdString(logo1);
  gotoXY(17,2);
  char logo2[13]={0x12,0x20,0x12,0x12,0x20,0x12,0x1d,0x20,0x12,0x12,0x20,0x16};
  lcdString(logo2);
  gotoXY(17,3);
  char logo3[13]={0x1d,0x13,0x16,0x12,0x20,0x12,0x12,0x19,0x12,0x12,0x15,0x12};
  lcdString(logo3);
  gotoXY(17,4);
  char logo4[13]={0x12,0x20,0x20,0x19,0x13,0x16,0x12,0x20,0x12,0x19,0x13,0x12};
  lcdString(logo4);
  gotoXY(6,6);
  lcdString("...on Arduino!");
  gotoXY(48,7);
  lcdString("2015-mia");
  delay(5000);
  lcdClear();
  lcdMapa();
  lcdScore(scorea,scoreb);
  palka1=constrain(map(analogRead(PIN_P1),0,1023,-52,104),1,52);
  palka2=constrain(map(analogRead(PIN_P2),0,1023,-52,104),1,52);
  int palkaold1=palka1;
  int palkaold2=palka2;
  micekX=5;
  micekY=palka1+4;
}

void loop(void) {
  if(ingame!=0) {
    ingame=lcdButton();
    }
  if(micekX+dX<0) {addscore(2);}
  if(micekY+dY<1||micekY+5+dY>64) {dY = -dY; tone(PIN_REPRO,1000,10);}
  if(micekX+5+dX>96) {addscore(1);}
  if(micekX+dX>2&&micekX+dX<5&&micekY+dY>=palka1-5&&micekY+dY<=palka1+12) {
    tone(PIN_REPRO,1200,10);
    switch(dY) {
      case 2:
        dX = -dX;
        if(micekY>palka1+3&&micekY<palka1+6) {dY=0;}
        if(micekY>=palka1-5&&micekY<=palka1+3) {dY=-dY;}
        break;
      case -2:
        dX = -dX;
        if(micekY>palka1+3&&micekY<palka1+6) {dY=0;}
        if(micekY<=palka1+12&&micekY>=palka1+6) {dY=-dY;}
        break;
      case 0:
        dX = -dX;
        if(micekY>=palka1-5&&micekY<=palka1+3) {dY=-2;}
        if(micekY<=palka1+12&&micekY>=palka1+6) {dY=2;}
        break;
      }
    }
  if(micekX+5+dX>91&&micekX+dX+5<93&&micekY+dY>=palka2-5&&micekY+dY<=palka2+12) {
    tone(PIN_REPRO,1200,10);
    switch(dY) {
      case 2:
        dX = -dX;
        if(micekY>palka2+3&&micekY<palka2+6) {dY=0;}
        if(micekY>=palka2-5&&micekY<=palka2+3) {dY=-dY;}
        break;
      case -2:
        dX = -dX;
        if(micekY>palka2+3&&micekY<palka2+6) {dY=0;}
        if(micekY<=palka2+12&&micekY>=palka2+6) {dY=-dY;}
        break;
      case 0:
        dX = -dX;
        if(micekY>=palka2-5&&micekY<=palka2+3) {dY=-2;}
        if(micekY<=palka2+12&&micekY>=palka2+6) {dY=2;}
        break;
      }
    }
  palka1=constrain(map(analogRead(PIN_P1),0,1023,-52,104),1,52);
  palka2=constrain(map(analogRead(PIN_P2),0,1023,-52,104),1,52);
  int palkaold1=palka1;
  int palkaold2=palka2;
  lcdPalka(palka1,palka2);
  lcdMicek(micekX,micekY);
  delay(12);
  lcdPalka(palkaold1,palkaold2);
  lcdMicek(micekX,micekY);
  if(ingame==0) {
  micekX += dX;
  micekY += dY;
  }
}

